@@ -43,34 +43,27 @@
 # include <Zend/zend_string.h>
 #endif
 
-#if PHP_VERSION_ID >= 50500
-#define ITERATOR_GET_CURRENT_KEY(it, z) \
-		(it).funcs->get_current_key(&(it), z TSRMLS_CC);
-#else
-#define ITERATOR_GET_CURRENT_KEY(it, z) \
-{ \
-	char *str_key; \
-	uint str_key_len; \
-	ulong int_key; \
-	switch ((it).funcs->get_current_key(&(it), &str_key, &str_key_len, &int_key TSRMLS_CC)) { \
-		case HASH_KEY_IS_LONG: \
-			ZVAL_LONG(z, int_key); \
-			break; \
-		case HASH_KEY_IS_STRING: \
-			ZVAL_STRINGL(z, str_key, str_key_len-1, 0); \
-			break; \
-		default: \
-			RETURN_NULL(); \
-	} \
-}
-#endif
-
 #if defined(__GNUC__)
 #  define ARG_UNUSED  __attribute__ ((unused))
 #else
 #  define ARG_UNUSED
 #endif
 
+#if defined (__GNUC__)
+# define PHONGO_GNUC_CHECK_VERSION(major, minor) \
+      ((__GNUC__ > (major)) ||                   \
+      ((__GNUC__ == (major)) && (__GNUC_MINOR__ >= (minor))))
+#else
+# define PHONGO_GNUC_CHECK_VERSION(major, minor) 0
+#endif
+
+#if PHONGO_GNUC_CHECK_VERSION(7, 0)
+# define PHONGO_BREAK_INTENTIONALLY_MISSING __attribute__ ((fallthrough));
+#else
+# define PHONGO_BREAK_INTENTIONALLY_MISSING
+#endif
+
+
 #if PHP_VERSION_ID >= 70000
 # define phongo_char zend_string
 # define phongo_long zend_long
@@ -150,22 +143,22 @@
 # define ADD_ASSOC_INT64(zval, key, value) add_assoc_long(zval, key, value)
 #elif SIZEOF_PHONGO_LONG == 4
 # define ADD_INDEX_INT64(zval, index, value) \
-    if (value > INT32_MAX || value < INT32_MIN) { \
-        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", value); \
+    if ((value) > INT32_MAX || (value) < INT32_MIN) { \
+        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", (value)); \
     } else { \
-        add_index_long(zval, index, value); \
+        add_index_long(zval, index, (value)); \
     }
 # define ADD_NEXT_INDEX_INT64(zval, value) \
-    if (value > INT32_MAX || value < INT32_MIN) { \
-        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", value); \
+    if ((value) > INT32_MAX || (value) < INT32_MIN) { \
+        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", (value)); \
     } else { \
-        add_next_index_long(zval, value); \
+        add_next_index_long(zval, (value)); \
     }
 # define ADD_ASSOC_INT64(zval, key, value) \
-    if (value > INT32_MAX || value < INT32_MIN) { \
-        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", value); \
+    if ((value) > INT32_MAX || (value) < INT32_MIN) { \
+        phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Integer overflow detected on your platform: %lld", (value)); \
     } else { \
-        add_assoc_long(zval, key, value); \
+        add_assoc_long(zval, key, (value)); \
     }
 #else
 # error Unsupported architecture (integers are neither 32-bit nor 64-bit)
