@@ -238,7 +238,7 @@ static void php_phongo_log(mongoc_log_level_t log_level, const char *log_domain,
 /* }}} */
 
 /* {{{ Init objects */
-static void phongo_cursor_init(zval *return_value, mongoc_client_t *client, mongoc_cursor_t *cursor, zval *readPreference TSRMLS_DC) /* {{{ */
+static void phongo_cursor_init(zval *return_value, mongoc_client_t *client, mongoc_cursor_t *cursor, zval *readPreference, zval *session TSRMLS_DC) /* {{{ */
 {
 	php_phongo_cursor_t *intern;
 
@@ -256,15 +256,24 @@ static void phongo_cursor_init(zval *return_value, mongoc_client_t *client, mong
 #else
 		Z_ADDREF_P(readPreference);
 		intern->read_preference = readPreference;
+#endif
+	}
+
+	if (session) {
+#if PHP_VERSION_ID >= 70000
+		ZVAL_ZVAL(&intern->session, session, 1, 0);
+#else
+		Z_ADDREF_P(session);
+		intern->session = session;
 #endif
 	}
 } /* }}} */
 
-static void phongo_cursor_init_for_command(zval *return_value, mongoc_client_t *client, mongoc_cursor_t *cursor, const char *db, zval *command, zval *readPreference TSRMLS_DC) /* {{{ */
+static void phongo_cursor_init_for_command(zval *return_value, mongoc_client_t *client, mongoc_cursor_t *cursor, const char *db, zval *command, zval *readPreference, zval *session TSRMLS_DC) /* {{{ */
 {
 	php_phongo_cursor_t *intern;
 
-	phongo_cursor_init(return_value, client, cursor, readPreference TSRMLS_CC);
+	phongo_cursor_init(return_value, client, cursor, readPreference, session TSRMLS_CC);
 	intern = Z_CURSOR_OBJ_P(return_value);
 
 	intern->database = estrdup(db);
@@ -277,11 +286,11 @@ static void phongo_cursor_init_for_command(zval *return_value, mongoc_client_t *
 #endif
 } /* }}} */
 
-static void phongo_cursor_init_for_query(zval *return_value, mongoc_client_t *client, mongoc_cursor_t *cursor, const char *namespace, zval *query, zval *readPreference TSRMLS_DC) /* {{{ */
+static void phongo_cursor_init_for_query(zval *return_value, mongoc_client_t *client, mongoc_cursor_t *cursor, const char *namespace, zval *query, zval *readPreference, zval *session TSRMLS_DC) /* {{{ */
 {
 	php_phongo_cursor_t *intern;
 
-	phongo_cursor_init(return_value, client, cursor, readPreference TSRMLS_CC);
+	phongo_cursor_init(return_value, client, cursor, readPreference, session TSRMLS_CC);
 	intern = Z_CURSOR_OBJ_P(return_value);
 
 	/* namespace has already been validated by phongo_execute_query() */
@@ -540,11 +549,13 @@ bool phongo_parse_read_preference(zval *options, zval **zreadPreference TSRMLS_D
 	return true;
 } /* }}} */
 
-/* Parses the "session" option for an execute method. If mongoc_opts is not
- * NULL, the option will be appended. If zsession is not NULL, it will be
+/* Parses the "session" option for an execute method. The client object should
+ * correspond to the Manager executing the operation and will be used to ensure
+ * that the session is correctly associated with that client. If mongoc_opts is
+ * not NULL, the option will be appended. If zsession is not NULL, it will be
  * assigned to the option. On error, false is returned and an exception is
  * thrown. */
-static bool phongo_parse_session(zval *options, bson_t *mongoc_opts, zval **zsession, mongoc_client_t *client TSRMLS_DC) /* {{{ */
+static bool phongo_parse_session(zval *options, mongoc_client_t *client, bson_t *mongoc_opts, zval **zsession TSRMLS_DC) /* {{{ */
 {
 	zval *option = NULL;
 	const mongoc_client_session_t *client_session;
@@ -652,7 +663,7 @@ bool phongo_execute_bulk_write(mongoc_client_t *client, const char *namespace, p
 		return false;
 	}
 
-	if (!phongo_parse_session(options, NULL, &zsession, client TSRMLS_CC)) {
+	if (!phongo_parse_session(options, client, NULL, &zsession TSRMLS_CC)) {
 		/* Exception should already have been thrown */
 		return false;
 	}
@@ -738,7 +749,7 @@ bool phongo_cursor_advance_and_check_for_error(mongoc_cursor_t *cursor TSRMLS_DC
 	return true;
 } /* }}} */
 
-int phongo_execute_query(mongoc_client_t *client, const char *namespace, zval *zquery, zval *options, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC) /* {{{ */
+bool phongo_execute_query(mongoc_client_t *client, const char *namespace, zval *zquery, zval *options, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC) /* {{{ */
 {
 	const php_phongo_query_t *query;
 	mongoc_cursor_t *cursor;
@@ -746,6 +757,7 @@ int phongo_execute_query(mongoc_client_t *client, const char *namespace, zval *z
 	char *collname;
 	mongoc_collection_t *collection;
 	zval *zreadPreference = NULL;
+	zval *zsession = NULL;
 
 	if (!phongo_split_namespace(namespace, &dbname, &collname)) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s: %s", "Invalid namespace provided", namespace);
@@ -767,7 +779,7 @@ int phongo_execute_query(mongoc_client_t *client, const char *namespace, zval *z
 		return false;
 	}
 
-	if (!phongo_parse_session(options, query->opts, NULL, client TSRMLS_CC)) {
+	if (!phongo_parse_session(options, client, query->opts, &zsession TSRMLS_CC)) {
 		/* Exception should already have been thrown */
 		mongoc_collection_destroy(collection);
 		return false;
@@ -797,7 +809,8 @@ int phongo_execute_query(mongoc_client_t *client, const char *namespace, zval *z
 		return true;
 	}
 
-	phongo_cursor_init_for_query(return_value, client, cursor, namespace, zquery, zreadPreference TSRMLS_CC);
+	phongo_cursor_init_for_query(return_value, client, cursor, namespace, zquery, zreadPreference, zsession TSRMLS_CC);
+
 	return true;
 } /* }}} */
 
@@ -814,7 +827,29 @@ static bson_t *create_wrapped_command_envelope(const char *db, bson_t *reply)
 	return tmp;
 }
 
-int phongo_execute_command(mongoc_client_t *client, php_phongo_command_type_t type, const char *db, zval *zcommand, zval *options, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC) /* {{{ */
+static zval *phongo_create_implicit_session(mongoc_client_t *client TSRMLS_DC) /* {{{ */
+{
+	mongoc_client_session_t *cs;
+	zval *zsession;
+
+	cs = mongoc_client_start_session(client, NULL, NULL);
+
+	if (!cs) {
+		return NULL;
+	}
+
+#if PHP_VERSION_ID >= 70000
+	zsession = ecalloc(sizeof(zval), 1);
+#else
+	ALLOC_INIT_ZVAL(zsession);
+#endif
+
+	phongo_session_init(zsession, cs TSRMLS_CC);
+
+	return zsession;
+} /* }}} */
+
+bool phongo_execute_command(mongoc_client_t *client, php_phongo_command_type_t type, const char *db, zval *zcommand, zval *options, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC) /* {{{ */
 {
 	const php_phongo_command_t *command;
 	bson_iter_t iter;
@@ -823,38 +858,51 @@ int phongo_execute_command(mongoc_client_t *client, php_phongo_command_type_t ty
 	bson_t opts = BSON_INITIALIZER;
 	mongoc_cursor_t *cmd_cursor;
 	zval *zreadPreference = NULL;
-	int result;
+	zval *zsession = NULL;
+	bool result = false;
+	bool free_reply = false;
+	bool free_zsession = false;
 
 	command = Z_COMMAND_OBJ_P(zcommand);
 
 	if ((type & PHONGO_OPTION_READ_CONCERN) && !phongo_parse_read_concern(options, &opts TSRMLS_CC)) {
 		/* Exception should already have been thrown */
-		bson_destroy(&opts);
-		return false;
+		goto cleanup;
 	}
 
 	if ((type & PHONGO_OPTION_READ_PREFERENCE) && !phongo_parse_read_preference(options, &zreadPreference TSRMLS_CC)) {
 		/* Exception should already have been thrown */
-		bson_destroy(&opts);
-		return false;
+		goto cleanup;
 	}
 
-	if (!phongo_parse_session(options, &opts, NULL, client TSRMLS_CC)) {
+	if (!phongo_parse_session(options, client, &opts, &zsession TSRMLS_CC)) {
 		/* Exception should already have been thrown */
-		bson_destroy(&opts);
-		return false;
+		goto cleanup;
+	}
+
+	/* If an explicit session was not provided, attempt to create an implicit
+	 * client session (ignoring any errors). */
+	if (!zsession) {
+		zsession = phongo_create_implicit_session(client TSRMLS_CC);
+
+		if (zsession) {
+			free_zsession = true;
+
+			if (!mongoc_client_session_append(Z_SESSION_OBJ_P(zsession)->client_session, &opts, NULL)) {
+				phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error appending implicit \"sessionId\" option");
+				goto cleanup;
+			}
+		}
 	}
 
 	if ((type & PHONGO_OPTION_WRITE_CONCERN) && !phongo_parse_write_concern(options, &opts, NULL TSRMLS_CC)) {
 		/* Exception should already have been thrown */
-		bson_destroy(&opts);
-		return false;
+		goto cleanup;
 	}
 
 	if (!BSON_APPEND_INT32(&opts, "serverId", server_id)) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Error appending \"serverId\" option");
-		bson_destroy(&opts);
-		return false;
+		goto cleanup;
 	}
 
 	/* Although "opts" already always includes the serverId option, the read
@@ -877,27 +925,25 @@ int phongo_execute_command(mongoc_client_t *client, php_phongo_command_type_t ty
 		default:
 			/* Should never happen, but if it does: exception */
 			phongo_throw_exception(PHONGO_ERROR_LOGIC TSRMLS_CC, "Type '%d' should never have been passed to phongo_execute_command, please file a bug report", type);
-			bson_destroy(&opts);
-			return false;
+			goto cleanup;
 	}
+
+	free_reply = true;
+
 	if (!result) {
 		phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
-		bson_destroy(&reply);
-		bson_destroy(&opts);
-		return false;
+		goto cleanup;
 	}
 
-	bson_destroy(&opts);
-
 	if (!return_value_used) {
-		bson_destroy(&reply);
-		return true;
+		goto cleanup;
 	}
 
 	/* According to mongoc_cursor_new_from_command_reply(), the reply bson_t
 	 * is ultimately destroyed on both success and failure. */
 	if (bson_iter_init_find(&iter, &reply, "cursor") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
 		bson_t initial_reply = BSON_INITIALIZER;
+		bson_error_t error = {0};
 
 		bson_copy_to(&reply, &initial_reply);
 
@@ -911,17 +957,39 @@ int phongo_execute_command(mongoc_client_t *client, php_phongo_command_type_t ty
 			bson_append_int64(&initial_reply, "batchSize", -1, command->batch_size);
 		}
 
+		if (zsession && !mongoc_client_session_append(Z_SESSION_OBJ_P(zsession)->client_session, &initial_reply, &error)) {
+			phongo_throw_exception_from_bson_error_t(&error TSRMLS_CC);
+			bson_destroy(&initial_reply);
+			result = false;
+			goto cleanup;
+		}
+
 		cmd_cursor = mongoc_cursor_new_from_command_reply(client, &initial_reply, server_id);
-		bson_destroy(&reply);
 	} else {
 		bson_t *wrapped_reply = create_wrapped_command_envelope(db, &reply);
 
 		cmd_cursor = mongoc_cursor_new_from_command_reply(client, wrapped_reply, server_id);
+	}
+
+	phongo_cursor_init_for_command(return_value, client, cmd_cursor, db, zcommand, zreadPreference, zsession TSRMLS_CC);
+
+cleanup:
+	bson_destroy(&opts);
+
+	if (free_reply) {
 		bson_destroy(&reply);
 	}
 
-	phongo_cursor_init_for_command(return_value, client, cmd_cursor, db, zcommand, zreadPreference TSRMLS_CC);
-	return true;
+	if (free_zsession) {
+#if PHP_VERSION_ID >= 70000
+		zval_ptr_dtor(zsession);
+		efree(zsession);
+#else
+		zval_ptr_dtor(&zsession);
+#endif
+	}
+
+	return result;
 } /* }}} */
 /* }}} */
 
